package ch.ethz.matsim.projects.astra_2018_002;

import java.io.File;

import org.matsim.api.core.v01.Scenario;
import org.matsim.core.config.Config;
import org.matsim.core.config.ConfigUtils;
import org.matsim.core.config.groups.StrategyConfigGroup.StrategySettings;
import org.matsim.core.controler.Controler;
import org.matsim.core.scenario.ScenarioUtils;

import ch.ethz.matsim.baseline_scenario.config.CommandLine;
import ch.ethz.matsim.baseline_scenario.config.CommandLine.ConfigurationException;
import ch.ethz.matsim.baseline_scenario.transit.BaselineTransitModule;
import ch.ethz.matsim.baseline_scenario.transit.routing.DefaultEnrichedTransitRoute;
import ch.ethz.matsim.baseline_scenario.transit.routing.DefaultEnrichedTransitRouteFactory;
import ch.ethz.matsim.projects.astra_2018_002.mode_choice.ModeChoiceModule;
import ch.sbb.matsim.routing.pt.raptor.SwissRailRaptorModule;

public class RunASTRA2018002 {
	static public void main(String[] args) throws ConfigurationException {
		CommandLine cmd = new CommandLine.Builder(args) //
				.requireOptions("config-path", "pt-pricing-path") //
				.allowPrefixes(ModeChoiceModule.SCORING_PREFIX) //
				.build();

		// Load config
		Config config = ConfigUtils.loadConfig(cmd.getOptionStrict("config-path"));

		config.strategy().clearStrategySettings();

		StrategySettings strategy = new StrategySettings();
		strategy.setStrategyName("custom");
		strategy.setWeight(0.2);
		config.strategy().addStrategySettings(strategy);

		strategy = new StrategySettings();
		strategy.setStrategyName("SubtourModeChoice");
		strategy.setWeight(0.0);
		strategy.setDisableAfter(1);
		config.strategy().addStrategySettings(strategy);

		strategy = new StrategySettings();
		strategy.setStrategyName("KeepLastSelected");
		strategy.setWeight(0.8);
		config.strategy().addStrategySettings(strategy);

		cmd.applyConfiguration(config);

		// Load scenario
		Scenario scenario = ScenarioUtils.createScenario(config);
		scenario.getPopulation().getFactory().getRouteFactories().setRouteFactory(DefaultEnrichedTransitRoute.class,
				new DefaultEnrichedTransitRouteFactory());
		ScenarioUtils.loadScenario(scenario);

		// Set up controller
		Controler controler = new Controler(scenario);
		controler.addOverridingModule(new SwissRailRaptorModule());
		controler.addOverridingModule(new BaselineTransitModule());

		File ptPricingPath = new File(cmd.getOptionStrict("pt-pricing-path"));
		controler.addOverridingModule(new ModeChoiceModule(ptPricingPath, cmd));

		// Run
		controler.run();
	}
}
