package ch.ethz.matsim.projects.astra_2018_002.mode_choice.estimation.car;

import java.util.List;

import org.matsim.api.core.v01.TransportMode;
import org.matsim.api.core.v01.network.Link;
import org.matsim.api.core.v01.network.Network;
import org.matsim.api.core.v01.population.Leg;
import org.matsim.api.core.v01.population.PlanElement;
import org.matsim.core.population.routes.NetworkRoute;
import org.matsim.core.router.TripRouter;
import org.matsim.core.router.TripStructureUtils.Trip;
import org.matsim.core.router.util.TravelTime;
import org.matsim.core.utils.geometry.CoordUtils;
import org.matsim.facilities.ActivityFacilities;
import org.matsim.facilities.ActivityFacility;

import ch.ethz.matsim.mode_choice.framework.ModeChoiceTrip;

public class CarPredictor {
	private final TripRouter router;
	private final ActivityFacilities facilities;
	private final Network network;
	private final TravelTime travelTimeCalculator;

	public CarPredictor(TripRouter router, ActivityFacilities facilities, TravelTime travelTimeCalculator,
			Network network) {
		this.router = router;
		this.facilities = facilities;
		this.travelTimeCalculator = travelTimeCalculator;
		this.network = network;
	}

	public CarPrediction predict(ModeChoiceTrip trip) {
		Trip tripInformation = trip.getTripInformation();

		ActivityFacility originFacility = facilities.getFacilities()
				.get(tripInformation.getOriginActivity().getFacilityId());
		ActivityFacility destinationFacility = facilities.getFacilities()
				.get(tripInformation.getDestinationActivity().getFacilityId());

		List<? extends PlanElement> result = router.calcRoute(TransportMode.car, originFacility, destinationFacility,
				tripInformation.getOriginActivity().getEndTime(), trip.getPerson());

		NetworkRoute route = (NetworkRoute) ((Leg) result.get(0)).getRoute();
		double travelDistance = route.getDistance();
		double travelTime = route.getTravelTime();

		// Also, while the NetworkRoutingModule does not take the arrival link into
		// account, it is simulated in the Netsim. Hence, to arrive at the correct
		// estimate, we need to add the travel time for the arrival link.
		double arrivalAtLinkTime = trip.getTripInformation().getOriginActivity().getEndTime() + travelTime;
		Link arrivalLink = network.getLinks().get(route.getEndLinkId());
		travelTime += travelTimeCalculator.getLinkTravelTime(arrivalLink, arrivalAtLinkTime, trip.getPerson(), null);

		double crowflyDistance = CoordUtils.calcEuclideanDistance(originFacility.getCoord(),
				destinationFacility.getCoord());

		return new CarPrediction(travelDistance, travelTime, crowflyDistance);
	}
}
