package ch.ethz.matsim.projects.astra_2018_002.mode_choice.estimation.other;

import java.util.List;

import org.matsim.api.core.v01.TransportMode;
import org.matsim.core.population.PersonUtils;

import ch.ethz.matsim.mode_choice.estimation.ModalTripEstimator;
import ch.ethz.matsim.mode_choice.estimation.TripCandidateWithPrediction;
import ch.ethz.matsim.mode_choice.framework.ModeChoiceTrip;
import ch.ethz.matsim.mode_choice.framework.trip_based.estimation.TripCandidate;
import ch.ethz.matsim.mode_choice.prediction.TeleportationPrediction;
import ch.ethz.matsim.mode_choice.prediction.TeleportationPredictor;
import ch.ethz.matsim.projects.astra_2018_002.mode_choice.estimation.ModeChoiceParameters;

public class BikeEstimator implements ModalTripEstimator {
	final private TeleportationPredictor predictor;
	final private ModeChoiceParameters parameters;

	public BikeEstimator(ModeChoiceParameters parameters, TeleportationPredictor predictor) {
		this.predictor = predictor;
		this.parameters = parameters;
	}

	@Override
	public TripCandidate estimateTrip(ModeChoiceTrip trip, List<TripCandidate> preceedingTrips) {
		TeleportationPrediction prediction = predictor.predict(trip);
		double travelTime_min = prediction.travelTime / 60.0;
		double age = 20; // PersonUtils.getAge(trip.getPerson());

		double utility = 0.0;
		utility += parameters.alphaBike;
		utility += parameters.betaTravelTimeBike_min * travelTime_min;
		utility += parameters.betaAgeBike_yr * Math.max(0.0, age - 18.0);

		return new TripCandidateWithPrediction(utility, TransportMode.bike, prediction);
	}
}
