package ch.ethz.matsim.projects.astra_2018_002.mode_choice.estimation.pt;

import java.util.List;

import org.matsim.api.core.v01.TransportMode;

import ch.ethz.matsim.mode_choice.estimation.ModalTripEstimator;
import ch.ethz.matsim.mode_choice.estimation.TripCandidateWithPrediction;
import ch.ethz.matsim.mode_choice.framework.ModeChoiceTrip;
import ch.ethz.matsim.mode_choice.framework.trip_based.estimation.TripCandidate;
import ch.ethz.matsim.projects.astra_2018_002.mode_choice.estimation.ModeChoiceParameters;

public class PublicTransportEstimator implements ModalTripEstimator {
	final private PublicTransportPredictor predictor;
	final private ModeChoiceParameters parameters;

	public PublicTransportEstimator(ModeChoiceParameters parameters, PublicTransportPredictor predictor) {
		this.predictor = predictor;
		this.parameters = parameters;
	}

	@Override
	public TripCandidate estimateTrip(ModeChoiceTrip trip, List<TripCandidate> preceedingTrips) {
		PublicTransportPrediction prediction = predictor.predict(trip);

		double inVehicleTime_min = prediction.inVehicleTime / 60.0;
		double transferTime_min = prediction.transferTime / 60.0;
		double accessEgressTime_min = prediction.accessEgressTime / 60.0;
		double inVehicleDistance_km = prediction.inVehicleDistance * 1e-3;

		double betaCost = parameters.betaCost(inVehicleDistance_km);

		double utility = 0.0;
		utility += parameters.alphaPublicTransport;
		utility += parameters.betaNumberOfTransfersPublicTransport * prediction.numberOfTransfers;

		utility += parameters.betaInVehicleTimePublicTransport_min * inVehicleTime_min;
		utility += parameters.betaTransferTimePublicTransport_min * transferTime_min;
		utility += parameters.betaAccessEgressTimePublicTransport_min * accessEgressTime_min;

		utility += betaCost * prediction.cost;

		if (prediction.crowflyDistance < parameters.shortDistance) {
			utility += parameters.shortDistancePenalty;
		}

		return new TripCandidateWithPrediction(utility, TransportMode.pt, prediction);
	}
}
